/*++

Copyright (c) 1990-2000 Microsoft Corporation, All Rights Reserved
 
Module Name:

    gpdread.c

Abstract:   Read a byte from a port using the genport driver.


Author:    Robert R. Howell January 6, 1993


Environment:

    User mode

Revision History:

    Robert B. Nelson (Microsoft) January 12, 1993


--*/



#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <winioctl.h>
#include "gpioctl.h"        // This defines the IOCTL constants.

#define GDP_MAX_BUFFER  0x100

void __cdecl main(int argc, char ** argv)
{

    // The following is returned by IOCTL.  It is true if the read succeeds.
    BOOL    IoctlResult;

    // The following parameters are used in the IOCTL call
    HANDLE  hndFile;            // Handle to device, obtain from CreateFile
    ULONG   PortNumber;         // Buffer sent to the driver (Port #).
    union   {
        ULONG   LongData;
        USHORT  ShortData;
        UCHAR   CharData;
    }   DataBuffer;             // Buffer received from driver (Data).
    LONG    IoctlCode;
    ULONG   DataLength;
    DWORD   ReturnedLength;     // Number of bytes returned
	UCHAR	TextBuffer[GDP_MAX_BUFFER];
	long lNumberOfBytesRead;

    // The input buffer is a ULONG containing the port address.  

    // The port data is returned in the output buffer DataBuffer;

    if ( argc < 3 || argv[1][0] != '-' ||
         ( argv[1][1] != 'b' && argv[1][1] != 'w' && argv[1][1] != 'd' && argv[1][1] != 't' ) )
    {
        printf("GpdRead -b|-w|-d|-t Port#  A byte (-b), word (-w), or a double\n");
        printf("                        word (-d) is read from the given port.\n");
        printf("                        Ports are numbered as 0, 1, ... relative\n");
        printf("                        to the base port assinged to the driver\n");
        printf("                        by PNP manager according to the driver preferences\n");
        printf("                        in the INF file. All numbers are read and printed in hex.\n");
        exit(1);
    }

    hndFile = CreateFile(
                "\\\\.\\GpdDev",                    // Open the Device "file"
                GENERIC_READ,
                FILE_SHARE_READ,
                NULL,
                OPEN_EXISTING,
                0,
                NULL
                );
                
    if (hndFile == INVALID_HANDLE_VALUE)        // Was the device opened?
    {
        printf("Unable to open the device.\n");
        exit(1);
    }

    switch (argv[1][1])
    {
        case 'b':
            IoctlCode = IOCTL_GPD_READ_PORT_UCHAR;
            DataLength = sizeof(DataBuffer.CharData);
            break;
            
        case 'w':
            IoctlCode = IOCTL_GPD_READ_PORT_USHORT;
            DataLength = sizeof(DataBuffer.ShortData);
            break;

        case 'd':
            IoctlCode = IOCTL_GPD_READ_PORT_ULONG;
            DataLength = sizeof(DataBuffer.LongData);
            break;

        case 't':
			break;

       default:
            printf("Invalid command line argument %c\n", argv[1][1]);
            exit(1);

    }

    sscanf(argv[2], "%x", &PortNumber);         // Get the port number to be read

	if (argv[1][1] == 't' ) {
		IoctlResult = ReadFile(
                        hndFile,                // Handle to device
                        TextBuffer,           // Buffer to driver.  Holds port & data.
                        GDP_MAX_BUFFER,             // Length of buffer in bytes.
                        &lNumberOfBytesRead,  // ǂݍ񂾃oCg
						NULL                    // NULL means wait till I/O completes.
						);
	}
	else {
	    IoctlResult = DeviceIoControl(
                            hndFile,            // Handle to device
                            IoctlCode,          // IO Control code for Read
                            &PortNumber,        // Buffer to driver.
                            sizeof(PortNumber), // Length of buffer in bytes.
                            &DataBuffer,        // Buffer from driver.
                            DataLength,         // Length of buffer in bytes.
                            &ReturnedLength,    // Bytes placed in DataBuffer.
                            NULL                // NULL means wait till op. completes.
                            );
	}
    if (IoctlResult)                            // Did the IOCTL succeed?
    {
        ULONG   Data;

        if(argv[1][1] == 't' ) {
			TextBuffer[lNumberOfBytesRead] = '\0'; // stop the read text

	        printf("Read from port %x returned [%s] (read length = %d)\n",
				PortNumber, TextBuffer, lNumberOfBytesRead);
		}
        else {
        if (ReturnedLength != DataLength)
        {
            printf(
                "Ioctl transfered %d bytes, expected %d bytes\n",
                ReturnedLength, DataLength );
        }
		switch (ReturnedLength)
        {
            case sizeof(UCHAR):
                Data = DataBuffer.CharData;
                break;
                
            case sizeof(USHORT):
                Data = DataBuffer.ShortData;
                break;
    
            case sizeof(ULONG):
                Data = DataBuffer.LongData;
                break;
            default:
                printf("Invalid return value from the DeviceIoControl\n");
                exit(1);

        }
        
        printf("Read from port %x returned %x\n", PortNumber, Data);
		} //else of (argv[1][1] == 't' )
    }
    else
    {
        printf("Ioctl failed with code %ld\n", GetLastError() );
    }


    if (!CloseHandle(hndFile))                  // Close the Device "file".
    {
        printf("Failed to close device.\n");
    }

    exit(0);
}

